@@ -50,9 +50,9 @@
 
 
 /* --- constants --- */
-static const gchar *CREATE_TITLES[] = { N_ ("Create Launcher"), N_ ("Create Link") };
-static const gchar *EDIT_TITLES[] = { N_ ("Edit Launcher"), N_ ("Edit Link") };
-static const gchar *ICON_NAMES[] = { "applications-other", "applications-internet" };
+static const gchar *CREATE_TITLES[] = { N_ ("Create Launcher"), N_ ("Create Link"), N_("Create Directory") };
+static const gchar *EDIT_TITLES[] = { N_ ("Edit Launcher"), N_ ("Edit Link"), N_("Edit Directory") };
+static const gchar *ICON_NAMES[] = { "applications-other", "applications-internet", "folder" };
 
 
 
@@ -69,6 +69,10 @@ static gint     opt_xid = 0;
 
 
 
+static void exo_die_error (const gchar *format, ...) G_GNUC_PRINTF (1, 2);
+
+
+
 /* --- command line options --- */
 static GOptionEntry option_entries[] =
 {
@@ -86,6 +90,24 @@ static GOptionEntry option_entries[] =
 
 
 
+static void
+exo_die_error (const gchar *format,
+               ...)
+{
+  gchar   *buffer;
+  va_list  args;
+
+  va_start (args, format);
+  buffer = g_strdup_vprintf (format, args);
+  va_end (args);
+
+  g_printerr ("%s: %s\n", G_LOG_DOMAIN, buffer);
+
+  g_free (buffer);
+}
+
+
+
 int
 main (int argc, char **argv)
 {
@@ -134,13 +156,13 @@ main (int argc, char **argv)
       if (G_UNLIKELY (error != NULL))
         {
           /* use the supplied error message */
-          g_critical ("%s", error->message);
+          exo_die_error ("%s", error->message);
           g_error_free (error);
         }
       else
         {
           /* no error message, the GUI initialization failed */
-          g_critical ("%s %s", _("Failed to open display"),
+          exo_die_error ("%s %s", _("Failed to open display"),
                       STR_FB (gdk_get_display_arg_name (), ""));
         }
 
@@ -167,16 +189,30 @@ main (int argc, char **argv)
   /* verify that a file/folder is specified */
   if (G_UNLIKELY (argc != 2))
     {
-      g_critical (_("No file/folder specified"));
+      exo_die_error (_("No file/folder specified"));
       return EXIT_FAILURE;
     }
 
+  /* create a file from the arguments */
+  gfile = g_file_new_for_commandline_arg (argv[1]);
+  if (!opt_create_new && opt_type == NULL
+      && !g_file_query_exists (gfile, NULL))
+    {
+      /* to help alacarte and some users a bit, we try to be smart here */
+      if (g_str_has_suffix (argv[1], ".desktop"))
+        opt_type = "Application";
+      else if (g_str_has_suffix (argv[1], ".directory"))
+        opt_type = "Directory";
+
+      /* go to create mode if we found a valid suffix */
+      opt_create_new = (opt_type != NULL);
+    }
+
   /* allocate a key file */
   key_file = g_key_file_new ();
-  gfile = g_file_new_for_commandline_arg (argv[1]);
 
   /* create new key file if --create-new was specified */
-  if (G_LIKELY (opt_create_new))
+  if (opt_create_new)
     {
       /* generic stuff */
       g_key_file_set_value (key_file, G_KEY_FILE_DESKTOP_GROUP,
@@ -195,8 +231,14 @@ main (int argc, char **argv)
                                 G_KEY_FILE_DESKTOP_KEY_ICON, STR_FB (opt_icon, "gnome-fs-bookmark"));
           g_key_file_set_value (key_file, G_KEY_FILE_DESKTOP_GROUP,
                                 G_KEY_FILE_DESKTOP_KEY_URL, STR_FB (opt_url, ""));
+
         }
-      else
+      else if (exo_str_is_equal (opt_type, G_KEY_FILE_DESKTOP_TYPE_DIRECTORY))
+        {
+          g_key_file_set_value (key_file, G_KEY_FILE_DESKTOP_GROUP,
+                                G_KEY_FILE_DESKTOP_KEY_ICON, STR_FB (opt_icon, ""));
+        }
+      else if (exo_str_is_equal (opt_type, G_KEY_FILE_DESKTOP_TYPE_APPLICATION))
         {
           g_key_file_set_value (key_file, G_KEY_FILE_DESKTOP_GROUP,
                                 G_KEY_FILE_DESKTOP_KEY_EXEC, STR_FB (opt_command, ""));
@@ -213,12 +255,12 @@ main (int argc, char **argv)
           /* we cannot open the file */
           if (G_LIKELY (error != NULL))
             {
-              g_critical (_("Failed to load contents from \"%s\": %s"), argv[1], error->message);
+              exo_die_error (_("Failed to load contents from \"%s\": %s"), argv[1], error->message);
               g_error_free (error);
             }
           else
             {
-              g_critical (_("The file \"%s\" contains no data"), argv[1]);
+              exo_die_error (_("The file \"%s\" contains no data"), argv[1]);
             }
 
           return EXIT_FAILURE;
@@ -231,7 +273,7 @@ main (int argc, char **argv)
       if (G_UNLIKELY (!res))
         {
           /* failed to parse the file */
-          g_critical (_("Failed to parse contents of \"%s\": %s"), argv[1], error->message);
+          exo_die_error (_("Failed to parse contents of \"%s\": %s"), argv[1], error->message);
           g_error_free (error);
           return EXIT_FAILURE;
         }
@@ -243,7 +285,7 @@ main (int argc, char **argv)
   if (G_UNLIKELY (value == NULL))
     {
       /* we cannot continue without a type */
-      g_critical (_("File \"%s\" has no type key"), argv[1]);
+      exo_die_error (_("File \"%s\" has no type key"), argv[1]);
       return EXIT_FAILURE;
     }
 
@@ -253,7 +295,7 @@ main (int argc, char **argv)
   if (G_UNLIKELY (enum_value == NULL))
     {
       /* tell the user that we don't support the type */
-      g_critical (_("Unsupported desktop file type \"%s\""), value);
+      exo_die_error (_("Unsupported desktop file type \"%s\""), value);
       return EXIT_FAILURE;
     }
   g_free (value);
@@ -335,6 +377,10 @@ main (int argc, char **argv)
       g_free (value);
       break;
 
+    case EXO_DIE_EDITOR_MODE_DIRECTORY:
+      /* nothing special */
+      break;
+
     default:
       g_assert_not_reached ();
       break;
@@ -397,13 +443,18 @@ main (int argc, char **argv)
                                  exo_die_editor_get_url (EXO_DIE_EDITOR (editor)));
           break;
 
+        case EXO_DIE_EDITOR_MODE_DIRECTORY:
+          /* nothing special */
+          break;
+
         default:
           g_assert_not_reached ();
           break;
         }
 
       /* try to save the file */
-      if (!exo_die_g_key_file_save (key_file, opt_create_new, gfile, &error) && opt_create_new)
+      if (!exo_die_g_key_file_save (key_file, opt_create_new, gfile, mode, &error)
+          && opt_create_new)
         {
           /* reset the error */
           g_clear_error (&error);
@@ -450,7 +501,7 @@ main (int argc, char **argv)
 
               /* try again to save to the new file */
               gfile = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (chooser));
-              exo_die_g_key_file_save (key_file, FALSE, gfile, &error);
+              exo_die_g_key_file_save (key_file, FALSE, gfile, mode, &error);
             }
 
           /* destroy the chooser */
